package com.jeeplus.test.cw.projectReport.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jeeplus.aop.logging.annotation.ApiLog;
import com.jeeplus.common.redis.RedisUtils;
import com.jeeplus.common.utils.ResponseUtil;
import com.jeeplus.sys.constant.enums.LogTypeEnum;
import com.jeeplus.sys.utils.UserUtils;
import com.jeeplus.test.cw.projectRecords.domain.CwProjectRecords;
import com.jeeplus.test.cw.projectRecords.service.dto.CwProjectRecordsDTO;
import com.jeeplus.test.cw.projectReport.domain.CwProjectReport;
import com.jeeplus.test.cw.projectReport.domain.CwProjectReportData;
import com.jeeplus.test.cw.projectReport.domain.CwProjectReportSignature;
import com.jeeplus.test.cw.projectReport.service.CwProjectReportService;
import com.jeeplus.test.cw.projectReport.service.dto.CwProjectReportDTO;
import com.jeeplus.test.cw.projectReport.service.dto.CwProjectReportSignatureDTO;
import com.jeeplus.test.cw.workClientInfo.service.dto.CwWorkClientBaseDTO;
import com.jeeplus.test.signature.utils.SignaturePostUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @author: 王强
 * @create: 2022-11-16 14:15
 **/
@RestController
@Api("财务报告信息")
@RequestMapping(value = "/cwProjectReport")
public class CwProjectReportController {

    @Resource
    private CwProjectReportService projectReportService;
    @Autowired
    private RedisUtils redisUtils;

    /**
     * 保存财务报告信息-流程
     * @param reportData
     * @return
     */
    @ApiLog(value = "保存财务报告信息-流程", type = LogTypeEnum.SAVE)
    @PreAuthorize("hasAnyAuthority('cwProjectReport:add','cwProjectReport:edit')")
    @PostMapping("saveForm")
    public ResponseEntity saveForm(@Valid @RequestBody CwProjectReportData reportData) throws Exception {
        String id = projectReportService.saveForm(reportData);
        return ResponseUtil.newInstance().add("businessTable", "cw_project_report").add("businessId", id).ok ("操作成功");
    }

    /**
     * 查询财务项目信息列表
     * @param projectReportData
     * @param page
     * @return
     */
    @ApiLog("查询财务报告信息列表")
    @PreAuthorize("hasAuthority('cwProjectReport:list')")
    @GetMapping("list")
    public ResponseEntity<IPage<CwProjectReportData>> data(CwProjectReportData projectReportData, Page<CwProjectReportData> page) throws Exception {
        IPage<CwProjectReportData> result = new Page<CwProjectReportData>();
        result = projectReportService.findList (page,projectReportData);
        return ResponseEntity.ok (result);
    }

    /**
     * 根据id修改状态值status
     * @param data
     * @return
     */
    @ApiOperation(value = "根据id修改状态值status")
    @PostMapping(value = "/updateStatusById")
    public ResponseEntity<String> updateStatusById(@RequestBody CwProjectReportData data) {
        String s = projectReportService.updateStatusById(data);
        return ResponseEntity.ok(s);
    }

    /**
     * 查询客户信息数据
     * @param id
     * @return
     */
    @ApiLog("查询财务报告数据")
    @PreAuthorize ("hasAnyAuthority('cwProjectReport:view','cwProjectReport:add','cwProjectReport:edit')")
    @GetMapping("queryById")
    public ResponseEntity queryById(@RequestParam("id") String id) {
        CwProjectReportData projectReportData = projectReportService.queryById ( id );
        return ResponseEntity.ok (projectReportData);
    }

    /**
     * 删除业务提问
     * @param ids
     * @return
     */
    @ApiLog(value = "删除财务报告信息", type = LogTypeEnum.SAVE)
    @PreAuthorize ("hasAuthority('cwProjectReport:del')")
    @DeleteMapping("delete")
    public ResponseEntity delete(String ids) {
        return projectReportService.deleteByIds(ids);
    }

    /**
     * 查询客户信息列表
     * @param cwWorkClientBaseDTO
     * @param page
     * @return
     * @throws Exception
     */
    @ApiLog("查询客户信息列表")
    @PreAuthorize("hasAuthority('cwProjectReport:clineList')")
    @GetMapping("clineList")
    public ResponseEntity<IPage<CwWorkClientBaseDTO>> data(CwWorkClientBaseDTO cwWorkClientBaseDTO, Page<CwWorkClientBaseDTO> page,String projectId) throws Exception {
        IPage<CwWorkClientBaseDTO> result = new Page<CwWorkClientBaseDTO>();
        result = projectReportService.findClineList (page,cwWorkClientBaseDTO,projectId);
        return ResponseEntity.ok (result);
    }

    /**
     * 签章流程审核通过
     * @param cwProjectReportData
     * @return
     */
    @ApiLog(value = "签章流程审核通过", type = LogTypeEnum.SAVE)
    @PostMapping("saveFlowableParam")
    public ResponseEntity<String> saveFlowableParam(@Valid @RequestBody CwProjectReportData cwProjectReportData) {
        CwProjectReportData cwProjectReportDto = projectReportService.queryById(cwProjectReportData.getId());
        // 获取签章页面url
        String mobile = UserUtils.getCurrentUserDTO().getMobile();
        String signatureContractUrl = SignaturePostUtil.getSignatureContractUrl(cwProjectReportDto.getSignatureContractId(), mobile);
        // 将数据存储到redis中，key为contractId
        cwProjectReportData.getCommitParamDTO().setVars(cwProjectReportDto);
        redisUtils.set("cw_report_signature_" + cwProjectReportDto.getSignatureContractId(),cwProjectReportData.getCommitParamDTO());
        redisUtils.expire("cw_report_signature_" + cwProjectReportDto.getSignatureContractId(),24, TimeUnit.HOURS);
        return ResponseEntity.ok(signatureContractUrl);
    }

    /**
     * 签章流程送审
     * @param cwProjectReportData
     * @return
     */
    @ApiLog(value = "签章流程送审", type = LogTypeEnum.SAVE)
    @PostMapping("saveData")
    public ResponseEntity<String> saveData(@Valid @RequestBody CwProjectReportData cwProjectReportData) {
        Map<String, Object> signatureContractId = projectReportService.createSignatureContractId(cwProjectReportData);
        boolean bool = (boolean)signatureContractId.get("success");
        if (bool) {
            String signatureId = projectReportService.getSignature(cwProjectReportData.getId());
            return ResponseUtil.newInstance().add("businessTable", "cw_project_report_signature").add("businessId", signatureId).ok ("操作成功");
        }
        return ResponseEntity.badRequest().body("操作失败");
    }

    /**
     * 根据签章表id查询报告数据
     * @param id
     * @return
     */
    @ApiLog("根据签章表id查询报告数据")
    @PreAuthorize ("hasAnyAuthority('cwProjectReport:view','cwProjectReport:add','cwProjectReport:edit')")
    @GetMapping("queryBySignatureId")
    public ResponseEntity queryBySignatureId(@RequestParam("id") String id) {
        CwProjectReportData projectReportData = projectReportService.queryBySignatureId ( id );
        return ResponseEntity.ok (projectReportData);
    }

    /**
     * 根据签章表id查询数据
     * @param id
     * @return
     */
    @ApiLog("根据签章表id查询数据")
    @PreAuthorize ("hasAnyAuthority('cwProjectReport:view','cwProjectReport:add','cwProjectReport:edit')")
    @GetMapping("findBySignatureId")
    public ResponseEntity findBySignatureId(@RequestParam("id") String id) {
        CwProjectReportSignature cwProjectReportSignature = projectReportService.findBySignatureId ( id );
        return ResponseEntity.ok (cwProjectReportSignature);
    }

    /**
     * 根据id修改签章表状态值status
     * @param data
     * @return
     */
    @ApiOperation(value = "根据id修改签章表状态值status")
    @PostMapping(value = "/updateSignatureStatusById")
    public ResponseEntity<String> updateSignatureStatusById(@RequestBody CwProjectReportSignatureDTO data) {
        String s = projectReportService.updateSignatureStatusById(data);
        return ResponseEntity.ok(s);
    }

    /**
     * 根据id修改签章表状态值status
     * @return
     */
    @ApiOperation(value = "根据id修改签章表状态值status")
    @PostMapping(value = "/aaa")
    public ResponseEntity<String> updateSignatureStatusById() {
        String s = "projectReportService.updateSignatureStatusById(data);";
        return ResponseEntity.ok(s);
    }
}
